function features = extract_features(frame)
  %extract the full 91-dimensional feature vector for a frame from the Camcube

  frameW = 200;
  frameH = 200;
  
  samplesPerFrame = frameW * frameH;
  
  features.featVersion = 1.4;
  numFeatures = 91;
  
  %add this variable which stores the width of the biggest filter used
  features.largestFilterSize = 1;
  features.x = zeros(samplesPerFrame, numFeatures);
  
  rows = repmat((1:frameH)', 1, frameW);
  cols = repmat(1:frameW, frameH, 1);
  rows = reshape(rows, samplesPerFrame, 1);
  cols = reshape(cols, samplesPerFrame, 1);
  
  radialDist = (rows - (frameH/2)).^2 + (cols-(frameW/2)).^2;
  radialDist = sqrt(radialDist);
  radialDist = reshape(radialDist, samplesPerFrame, 1);
  
  features.x(:, 1) = radialDist;
  features.descrip{1} = 'radial dist';
  features.normaliseGroups(1) = 1;
  nextFeature = 2;
  
  %get the different processing we are doing to amplitude, store it, makes sure it is normalised as one
  [ampData ampDescrip] = ...
      getMinMax(frame.amplitudes, samplesPerFrame, 'amplitude');
  featRange = nextFeature:(nextFeature+size(ampData, 2)-1);
  features.x(:, featRange) = ampData;
  features.descrip(featRange) = ampDescrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  nextFeature = featRange(end)+1;
  
  %intensities
  [intData intDescrip] = ...
      getMinMax(frame.intensities, samplesPerFrame, 'intensities');
  featRange = nextFeature:(nextFeature+size(intData, 2)-1);
  features.x(:, featRange) = intData;
  features.descrip(featRange) = intDescrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  nextFeature = featRange(end)+1;
  
  %distances
  [distData distDescrip] = ...
      getMinMax(frame.dists, samplesPerFrame, 'distances');
  featRange = nextFeature:(nextFeature + size(distData, 2)-1);
  features.x(:, featRange) = distData;
  features.descrip(featRange) = distDescrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  nextFeature = featRange(end)+1;
  
  %amplitude - 3 radius first
  [amp3Data amp3Descrip] = ...
      getMinMax(laplacian_filter(frame.amplitudes, 3), samplesPerFrame, 'lap amp rad 3');
  featRange = nextFeature:(nextFeature + size(amp3Data, 2)-1);
  features.x(:, featRange) = amp3Data;
  features.descrip(featRange) = amp3Descrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  nextFeature = featRange(end)+1;
  
  [amp5Data amp5Descrip] = ...
      getMinMax(laplacian_filter(frame.amplitudes, 5), samplesPerFrame, 'lap amp rad 5');
  featRange = nextFeature:(nextFeature + size(amp5Data, 2)-1);
  features.x(:, featRange) = amp5Data;
  features.descrip(featRange) = amp5Descrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  nextFeature = featRange(end)+1; 
  
  %laplacians - 3 radius first
  [lap3Data lap3Descrip] = ...
      getMinMax(laplacian_filter(frame.dists, 3), samplesPerFrame, 'lap dist rad 3');
  featRange = nextFeature:(nextFeature + size(lap3Data, 2) - 1);
  features.x(:, featRange) = lap3Data;
  features.descrip(featRange) = lap3Descrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  features.largestFilterSize = max(features.largestFilterSize, 3);
  nextFeature = featRange(end) + 1;
  
  %laplacian radius 5
  [lap5Data lap5Descrip] = ...
      getMinMax(laplacian_filter(frame.dists, 5), samplesPerFrame, 'lap dist rad 5');
  featRange = nextFeature:(nextFeature + size(lap5Data, 2) - 1);
  features.x(:, featRange) = lap5Data;
  features.descrip(featRange) = lap5Descrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  features.largestFilterSize = max(features.largestFilterSize, 5);
  nextFeature = featRange(end) + 1;
  
  %laplacian radius 7
  [lap7Data lap7Descrip] = ...
      getMinMax(laplacian_filter(frame.dists, 7), samplesPerFrame, 'lap dist rad 7');
  featRange = nextFeature:(nextFeature + size(lap7Data, 2) - 1);
  features.x(:, featRange) = lap7Data;
  features.descrip(featRange) = lap7Descrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  features.largestFilterSize = max(features.largestFilterSize, 7);
  nextFeature = featRange(end) + 1;
  
  
  
  %normals in X
  [normXData normXDescrip] = ...
      getMinMax(get_normals_x(frame.dists), samplesPerFrame, 'normX');
  featRange = nextFeature:(nextFeature + size(normXData, 2) - 1);
  features.x(:, featRange) = normXData;
  features.descrip(featRange) = normXDescrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  nextFeature = featRange(end) + 1;
  
  %normals in Y
  [normYData normYDescrip] = ...
      getMinMax(get_normals_y(frame.dists), samplesPerFrame, 'normY');
  featRange = nextFeature:(nextFeature + size(normYData, 2) - 1);
  features.x(:, featRange) = normYData;
  features.descrip(featRange) = normYDescrip;
  features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
  nextFeature = featRange(end) + 1;
  
  %gabor wavelets on intensity and distance first - 4 orientations - setup some parameters for a 13x13 - this looses a bunch of data but whatever
  sigma = 2;
  lambda = 1.8;
  psi = 0;
  gamma = 1;
  orientations = [0 pi/4 pi/2 3*pi/4];
  names = {'gabor %s angle 0', 'gabor %s angle pi/4', ...
           'gabor %s angle pi/2', 'gabor %s angle 3*pi/4'};
  
  for i=1:length(orientations),
    %intensity
    gaborFilter = gabor_wavelet(sigma, orientations(i), lambda, psi, gamma);
    [gaborData gaborDescrip] = ...
        getMinMax(filter2(gaborFilter, frame.intensities), samplesPerFrame, ...
                                sprintf(names{i}, 'intensity'));
    featRange = nextFeature:(nextFeature + size(gaborData, 2) - 1);
    features.x(:, featRange) = gaborData;
    features.descrip(featRange) = gaborDescrip;
    features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
    features.largestFilterSize = max(features.largestFilterSize, size(gaborFilter, 1));
    nextFeature = featRange(end) + 1;
    
    %distance
    [gaborData gaborDescrip] = ...
        getMinMax(filter2(gaborFilter, frame.dists), samplesPerFrame, ...
                                sprintf(names{i}, 'distances'));
    featRange = nextFeature:(nextFeature + size(gaborData, 2) - 1);
    features.x(:, featRange) = gaborData;
    features.descrip(featRange) = gaborDescrip;
    features.normaliseGroups(featRange) = max(features.normaliseGroups) + 1;
    features.largestFilterSize = max(features.largestFilterSize, size(gaborFilter, 1));
    nextFeature = featRange(end) + 1;
  end 
  
  
  
end

function [f descrip] = getMinMax(data, samplesPerFrame, name)

  NUM_FEATURES = 5;
  f = zeros(samplesPerFrame, NUM_FEATURES);
  descrip = cell(1, NUM_FEATURES);
  
  [dataMin dataMax dataMean dataMedian] = getAllAverages(data);
  
  f(:, 1) = reshape(data, samplesPerFrame, 1);
  f(:,2:5) = repmat([dataMin dataMax dataMean dataMedian], samplesPerFrame, 1);
  
  descrip{1} = name;
  descrip{2} = sprintf('%s min', name);
  descrip{3} = sprintf('%s max', name);
  descrip{4} = sprintf('%s mean', name);
  descrip{5} = sprintf('%s median', name);

end

function [minD maxD meanD medianD] = getAllAverages(data)
  minD = min(data(:));
  maxD = max(data(:));
  medianD = median(data(:));
  meanD = mean(data(:));
end
